#ifndef __BGIGRAPH_H_
#define __BGIGRAPH_H_

#include <alloc.h>
#include <graphics.h>
#include <stdlib.h>
#include <string.h>

#include <defines.h>
#include <cscreen.h>

#define FULL_SCR			CRect<int> (0, 0, getmaxx(), getmaxy())

extern void stopBGIgraph();
extern int startBGIgraph  (int gDrive = DETECT, int gMode = DETECT);

class CBGIScreen: public CScreen
{
private:
					CRect<int> myVPLoc;
					void LeftTopRectangle   (CRect<int>  aLoc, int aColor);
					void RightDownRectangle (CRect<int>  aLoc, int aColor);
					int myCharWidth;
public:
//-------------------CONSTRUCTORS--------------------------------------------
					CBGIScreen (int gDrive = DETECT, int gMode = DETECT);
					virtual ~CBGIScreen();
//--------------------------------------------------------------------------
					virtual int frontcolor	();
					virtual int backcolor	();
					virtual void frontcolor	(int aColor);
					virtual void backcolor	(int aColor);

					virtual int  font		();
					virtual int  fontsize	();
					virtual void font		(int aFont);
					virtual void fontsize	(int aSize);

					virtual int  widthoftext	(char *aStr);
					virtual int  heightoftext	(char *aStr);

					virtual void point 	  	(CPoint<int>  aLoc,
											 int aFrontcolor = NON);

					virtual void Line 	  	(CRect<int>  aLoc,
											 int aFrontcolor = NON);

					virtual void rect 	    (CRect<int>  aLoc,
											 int aBackcolor = NON,
											 int aFrontcolor = NON);

					virtual void text 	    (CPoint<int>  aLoc,
											 char *aText,
											 int aFrontcolor = NON,
											 int aBackcolor = NON);

					virtual void rect3d (CRect<int>  aLoc, int status,
								 int aColor = NON);

					virtual void fillstyle (int aPattern = SOLID_FILL,
											int aColor = LIGHTGRAY);

					virtual void fillback (int aBackcolor = LIGHTGRAY);

					virtual void textstyle (int aFont = DEFAULT_FONT,
											int aSize = 1);

					virtual int maxx();
					virtual int maxy();
					virtual int maxz();

					virtual void viewport (CRect<int>  aLoc = FULL,
										   int aClip = TRUE);

					virtual void saveviewport();
					virtual void loadviewport();
					virtual void addviewport (CRect<int> aLoc);
					virtual CRect<int> getviewport();

					virtual void clip            (int aClip);
};
//-------------------CONSTRUCTORS--------------------------------------------
CBGIScreen::CBGIScreen (int gDrive, int gMode)
{
myFrontcolor = WHITE;
myBackcolor  = LIGHTGRAY;
myFillstyle	 = SOLID_FILL;
if (startBGIgraph (gDrive, gMode)) {printf ("Can't init graphics"); exit(1);}
myCharWidth  = textwidth ("A");

}

CBGIScreen::~CBGIScreen()
{
stopBGIgraph();
}

//----------------------------------DRAW-------------------------------------

void CBGIScreen::text (CPoint<int>  aLoc, char *aText,
					   int aFrontcolor, int aBackcolor)
{
int len = strlen (aText);
if (aBackcolor != NON)
	{
	setcolor (aBackcolor);
	for (int i = 0; i < len; i++)
		{
		char tmp[2] = {aText[i]};
		outtextxy (aLoc.x + i * myCharWidth + 1, aLoc.y + 1, tmp);
		}
	}
if (aFrontcolor != NON) setcolor (aFrontcolor);
if (aFrontcolor == NON && myFrontcolor != NON) 	setcolor (myFrontcolor);
for (int i = 0; i < len; i++)
	{
	char tmp[2] = {aText[i]};
	outtextxy (aLoc.x + i * myCharWidth, aLoc.y, tmp);
	}
}

void CBGIScreen::point (CPoint<int>  aLoc, int aFrontcolor)
{
if (aFrontcolor != NON)	putpixel (aLoc.x, aLoc.y, aFrontcolor);
if (aFrontcolor == NON && myFrontcolor != NON) putpixel (aLoc.x, aLoc.y, myFrontcolor);
}


void CBGIScreen::Line (CRect<int>  aLoc, int aFrontcolor)
{

if (aFrontcolor != NON)
	{

	setcolor (aFrontcolor);
	line (aLoc.x(), aLoc.y(), aLoc.x2(), aLoc.y2());
	}

if (aFrontcolor == NON && myFrontcolor != NON)    //!!!!!!!!!!!!
	{
	setcolor (myFrontcolor);
	line (aLoc.x(), aLoc.y(), aLoc.x2(), aLoc.y2());
	}
}

void CBGIScreen::rect (CRect<int>  aLoc, int aBackcolor, int aFrontcolor)
{
if (aBackcolor != NON)
	{
	setfillstyle (myFillstyle, aBackcolor);
	bar (aLoc.x(), aLoc.y(), aLoc.x2(), aLoc.y2());
	}

if (aFrontcolor != NON)
	{
	setcolor (aFrontcolor);
	rectangle (aLoc.x(), aLoc.y(), aLoc.x2(), aLoc.y2());
	}
}
//----------------------OTHER------------------------------------------------
void CBGIScreen::fillstyle (int aPattern, int aColor)
{
myFillstyle	= aPattern;
myBackcolor = aColor;
}

void CBGIScreen::fillback (int aBackcolor)
{
rect (CRect<int>  (0, 0, maxx(), maxy()), aBackcolor, aBackcolor);
}

int CBGIScreen::maxx()
{
return getmaxx();
}

int CBGIScreen::maxy()
{
return getmaxy();
}

int CBGIScreen::maxz()
{
return FALSE;//!!!!!!!!!!!!!!!!
}

int CBGIScreen::frontcolor()
{
return myFrontcolor;
}

int CBGIScreen::backcolor()
{
return myBackcolor;
}

void CBGIScreen::frontcolor (int aColor)
{
myFrontcolor  = aColor;
}

void CBGIScreen::backcolor (int aColor)
{
myBackcolor  = aColor;
}

void CBGIScreen::font (int aFont)
{
myFont = aFont;
settextstyle (myFont, HORIZ_DIR, myFontSize);
myCharWidth  = textwidth ("");
}

int CBGIScreen::font()
{
return myFont;
}

void CBGIScreen::fontsize (int aSize)
{
myFontSize = aSize;
settextstyle (myFont, HORIZ_DIR, myFontSize);
myCharWidth  = textwidth ("A");
}

int CBGIScreen::fontsize()
{
return myFontSize;
}

int CBGIScreen::widthoftext (char *aStr)
{
return textwidth (aStr);
}

int CBGIScreen::heightoftext (char *aStr)
{
return textheight (aStr);
}

void CBGIScreen::textstyle (int aFont, int aSize)
{
myFont 	   = aFont;
myFontSize = aSize;
settextstyle (myFont, HORIZ_DIR, myFontSize);
myCharWidth  = textwidth ("A");
}

//---------------------------------------------------------------------------
void CBGIScreen::viewport (CRect<int>  aLoc, int aClip)
{
if (!aLoc)
	{
	myVPLoc = (CRect<int> (0, 0, maxx(), maxy()));
	setviewport (0, 0, maxx(), maxy(), 1);
	}
else
	{
	int x  = aLoc.x(),  y  = aLoc.y();
	int x2 = aLoc.x2(), y2 = aLoc.y2();
	if (x  < 0) x = 0;
	if (y  < 0) y = 0;
	if (x2 >= maxx()) x2 = maxx() - 1;
	if (y2 >= maxy()) y2 = maxy() - 1;

	setviewport (x, y, x2, y2, aClip);
	myVPLoc = aLoc;
	}
}

void CBGIScreen::saveviewport()
{
viewporttype port;
getviewsettings (&port);
mySavedVP.x  (port.left);
mySavedVP.y  (port.top);
mySavedVP.x2 (port.right);
mySavedVP.y2 (port.bottom);
}

void CBGIScreen::loadviewport()
{
setviewport (mySavedVP.x(), mySavedVP.y(), mySavedVP.x2(), mySavedVP.y2(), 1);
}

void CBGIScreen::addviewport (CRect<int> aLoc)
{
if (aLoc.x() > myVPLoc.width() || aLoc.y() > myVPLoc.height()) return;
int aX2 = aLoc.x2(), aY2 = aLoc.y2();
if (aX2 > myVPLoc.width())  aX2 = myVPLoc.width();
if (aY2 > myVPLoc.height()) aY2 = myVPLoc.height();
myVPLoc = CRect<int> (myVPLoc.x() + aLoc.x(), myVPLoc.y() + aLoc.y(),
									 aX2 - aLoc.x(), aY2 - aLoc.y());
viewport (myVPLoc, 1);
}

CRect<int> CBGIScreen::getviewport()
{
viewporttype p;
getviewsettings (&p);
return CRect<int> (p.left, p.top, p.right - p.left, p.bottom - p.top);
}

void CBGIScreen::clip (int aClip)
{
viewport  (myVPLoc, aClip);
}
//---------------------------PSEVDO-3D--------------------------------------\\
//------------------------------rect3D--------------------------------------\\

void CBGIScreen::rect3d (CRect<int>  aLoc, int status, int aColor)
{
if (aColor == NON) aColor = myBackcolor;
if (status != HIDE) rect (aLoc, aColor, aColor);

switch (status)
	{
	case HIDE:    break;
	case UNPRESS: LeftTopRectangle   (aLoc, WHITE);
				  RightDownRectangle (aLoc, BLACK);
				  break;

	case PRESS:	  LeftTopRectangle   (aLoc, BLACK);
				  RightDownRectangle (aLoc, WHITE);
				  break;

	case DISABLE: LeftTopRectangle   (aLoc, WHITE);
				  RightDownRectangle (aLoc, DARKGRAY);
				  break;
	}
}

void CBGIScreen::LeftTopRectangle   (CRect<int>  aLoc, int aColor)
{
Line (aLoc.left(), aColor);
Line (aLoc.top(),  aColor);
}

void CBGIScreen::RightDownRectangle (CRect<int>  aLoc, int aColor)
{
Line (aLoc.right(),  aColor);
Line (aLoc.bottom(), aColor);
}
//==========================================================================

int startBGIgraph  (int gDrive, int gMode)
{
initgraph (&gDrive, &gMode, getenv("BGI"));
return graphresult();
}

void stopBGIgraph()
{
closegraph();
}
#endif
