#ifndef _CBUTTON_H
#define _CBUTTON_H

#include <cmouse.h>
#include <colw.h>

#define BUTTON_CAPTION_LONG  		32
//-----------------------------CBUTTON--------------------------------------
class CButton: public CWindow
{
protected:		char myCaption[BUTTON_CAPTION_LONG];

				int  myClickStat;

				void (*myMsP)(CPoint<int> aLoc, CButton *w);
				void (*myKbP)(int aKey, CButton *w);
				void (*myGetFocusP) (CButton *b);
				void (*myLostFocusP) (CButton *b);
				void (*myMsClickP) (CButton *b);

				virtual void outtext();
				virtual void click  (int aStat, CPoint<int> aLoc = (0, 0, 0, 0));
public:
				CButton (CRect<int> aLoc, char *aCaption,
						 void (*aMsClickP)(CButton *b) = NULL,
						 void (*aKbP)(int aKey, CButton *b) = NULL);
				CButton ();
				virtual ~CButton ();

				virtual void draw();
				virtual void caption  (char *aCap);
				virtual char *caption ();
				virtual void getFocus ();
				virtual void lostFocus();
				virtual void onMouse  (CPoint<int> aLoc);
				virtual void offMouse ();
				virtual void onKbhit  (int aKey);
				virtual void viewport ();
//---------------------------------------------------------------------------
				virtual void getFocusP  (void (*p) (CButton *b));
				virtual void lostFocusP (void (*p) (CButton *b));
				virtual void OnMouseP (void (*p) (CPoint<int> aLoc,
													CButton *b));
};
//----------------CONSTRUCTORS----------------------------------------------
CButton::CButton (CRect<int> aLoc, char *aCaption,
				  void (*aMsClickP)(CButton *b),
				  void (*aKbP)(int aKey, CButton *b))
{
myClickStat  = FALSE;
myLoc 		 = aLoc;
myMsClickP   = aMsClickP;
myKbP 		 = aKbP;
myMsP		 = NULL;
myGetFocusP  = NULL;
myLostFocusP = NULL;
strncpy (myCaption, aCaption, BUTTON_CAPTION_LONG);
}

CButton::CButton()
{
myClickStat  = FALSE;
myKbP        = NULL;
myMsP		 = NULL;
myGetFocusP  = NULL;
myLostFocusP = NULL;
myMsClickP   = NULL;
strncpy (myCaption, "Button", BUTTON_CAPTION_LONG);
}

CButton::~CButton()
{
}
//---------PROTECTED---------------------------------------------------------
void CButton::outtext()
{
if (!myDc) return;
viewport();
myDc->addviewport (myLoc);
myDc->textstyle (DEFAULT_FONT, 1);
switch (myStatus)
	{
	case HIDE:
		myDc->text (CPoint<int> (5, 7), myCaption,  LIGHTGRAY);		 break;
	case PRESS:
		myDc->text (CPoint<int> (5, 7), myCaption,  BLACK);			 break;
	case UNPRESS:
		myDc->text (CPoint<int> (5, 7), myCaption, BLACK); 			 break;
	case DISABLE:
		myDc->text (CPoint<int> (5, 7), myCaption, DARKGRAY, WHITE); break;
	}
}
//---------PROTECTED---------------------------------------------------------

void CButton::draw()
{
if (!myDc) return;
myMouse->hide();
viewport();
myDc->rect3d (myLoc, myStatus);
outtext();
myMouse->show();
}

void CButton::caption (char *aCap)
{
strncpy (myCaption, aCap, BUTTON_CAPTION_LONG);
draw();
}

char *CButton::caption()
{
return myCaption;
}

//---------------------------------------------------------------------------
void CButton::getFocus()
{
if (!myDc) return;
myMouse->hide();
viewport();
myDc->rect (myLoc.grow(1), NON, BLACK);
myMouse->show();
if (myGetFocusP) (*myGetFocusP) (this);
}

void CButton::lostFocus()
{
if (!myDc) return;
myMouse->hide();
viewport();
myDc->rect (myLoc.grow(1), NON, LIGHTGRAY);
myMouse->show();
if (myLostFocusP) (*myLostFocusP) (this);
}

//---------------------------------------------------------------------------
void CButton::onMouse (CPoint<int> aLoc)
{
if (!myMouse->move() && myStatus != HIDE)
	{
	click (TRUE, aLoc);
	if (myMsP) (*myMsP)(aLoc, this);
	}
}

void CButton::offMouse ()
{
if (myStatus != HIDE)
	click (FALSE);
}

void CButton::viewport()
{
if (myPar) myPar->viewport();
}

void CButton::click (int aStat, CPoint<int> aLoc)
{
aLoc++;
if (aStat == FALSE)
	{
	if (myStatus != UNPRESS) status (UNPRESS);
	myClickStat = FALSE;
	}
if (myMouse->whatOn() == LEFTB && !myClickStat)
	{
	status (PRESS);
	myClickStat = TRUE;
	}
if (myMouse->whatOff() == LEFTB && myClickStat == TRUE)
	{
	status (UNPRESS);
	myClickStat = FALSE;
	if (myMsClickP) (*myMsClickP)(this);
	}
}
//---------------------------------------------------------------------------
void CButton::onKbhit (int aKey)
{
if (myKbP) (*myKbP) (aKey, this);
if (aKey == ENTER && myMsClickP) (*myMsClickP)(this);
}

void CButton::getFocusP (void (*p) (CButton *b))
{
myGetFocusP = p;
}

void CButton::lostFocusP (void (*p) (CButton *b))
{
myLostFocusP = p;
}

void CButton::OnMouseP (void (*p)(CPoint<int> aLoc, CButton *b))
{
myMsP = p;
}
#endif



