#ifndef __CCAR_H__
#define __CCAR_H__

#include <bios.h>
#include <stdlib.h>
#include <conio.h>
#include <stdio.h>
#include <time.h>
#include <alloc.h>

#include <crect.h>
#include <cbgi.h>
#include <defines.h>
#include <keys.h>

#include <ccarmask.h>
#include <croad.h>
#include <cpark.h>

#define				MAXSPEED			140
#define				MINSPEED			10
#define				ENDMOVE				0x0A
#define             END_RUN             0x0B
#define				HIDE                0
#define				DRAW                1



//-------------------CCar--------------------------------------------------\\
class CCar
{
private:
				int           myBackCol;
				int           myStop;
				int           myCurGBDD;
				float     	  mySpeed;
				float      	  myWay;
				float 	      myKof;
				clock_t 	  myPrevT;
				CPoint<float> myP;
				CRoad        *myRoad;
				CCarMask     *myMask;
				CScreen      *myDc;
				CPark        *myParking;

				void    	 myDraw (int aParam);
public:
				CCar (CScreen *aDc, CRoad *aRoad, CCarMask *aMask,
								 float aK, int aBk, CPark *aPark);
				CCar ();
				~CCar();

				CRoad *road();
				int    move();
				void   prepare();
				void   kof (float aKof);
				void   resetall (CRoad *aRoad, CCarMask *aMask);
};

//-------------CONSTRUCTORS------------------------------------------------\\
CCar::CCar (CScreen *aDc, CRoad *aRoad, CCarMask *aMask,
						float aK, int aBk, CPark *aPark)
{
myParking = aPark;
myCurGBDD = 0;
myBackCol = aBk;
myDc      = aDc;
myStop    = TRUE;
myKof     = aK;
myMask    = aMask;
myWay     = 0;
myRoad    = aRoad;
mySpeed   = random (MAXSPEED - MINSPEED) + MINSPEED;
}

CCar::CCar()
{
myCurGBDD = 0;
myBackCol = BLACK;
myStop    = TRUE;
myKof     = 0.1;
myWay     = 0;
myMask    = NULL;
myRoad    = NULL;
myParking = NULL;
mySpeed   = random (MAXSPEED - MINSPEED) + MINSPEED;
}

CCar::~CCar()
{
}

//-------------FUNCTIONS---------------------------------------------------\\
CRoad *CCar::road()
{
return myRoad;
}

void CCar::kof (float aKof)
{
myKof = aKof;
}

//ENTRY:  void;
//RETURN: status of moving;
int CCar::move()
{
float myPrevPos     = myWay;
clock_t myTime      = clock();
if (!myStop && !myKof) myStop  = TRUE;
if (myStop  &&  myKof) myStop  = FALSE;
if (myStop  && !myKof) myPrevT = myTime;
myWay  += (myTime - myPrevT) * mySpeed * myKof;
if (myWay == myPrevPos) return FALSE;
myPrevT = myTime;
myDraw (HIDE);
myP = (myRoad->getpos (myWay));
if (myRoad->testGBDD (myWay, mySpeed, myCurGBDD))
	{
	myParking->addcar (myMask);
	return ENDMOVE;
	}
if (myP == CPoint<float> (0xFF, 0xFF, 0xFF)) 	  return ENDMOVE;
myDraw (DRAW);
return TRUE;
}

//ENTRY:  pointers to roads and car masks;
//RETURN: void;
void CCar::resetall (CRoad *aRoad, CCarMask *aMask)
{
myCurGBDD = 0;
myP       = 0;
myStop    = TRUE;
myWay     = 0;
myMask    = aMask;
myRoad    = aRoad;
mySpeed   = random (MAXSPEED - MINSPEED) + MINSPEED;
}

//ENTRY:  void;
//RETURN: void;
void CCar::prepare()
{
myPrevT = clock();
myStop  = FALSE;
}

//ENTRY:  DRAW or HIDE;
//RETURN: void;
void CCar::myDraw (int aParam)
{
if (aParam == HIDE) myRoad->draw();
myMask->draw (myP, aParam, myBackCol);
}

//--------------CCarManager------------------------------------------------\\

class CGBDD
{
private:
				int     	  myRoadNum;
				int           myMaskNum;
				int           myBackCol;
				float         myKof;
				CPoint<float> myLoc;
				CCarMask    **myMask;
				CCar  		**myCar;
				CRoad 		**myRoad;
				CScreen      *myDc;
				CPark        *myParking;

				void    myNewKof();
				void    myDraw();
public:
				CGBDD (int aRoadNum,   CRoad **aRoad,
					   int aMaskNum, CCarMask **aMask,
					   CPoint<float> aLoc, float aKof, CScreen *aDc, int aBk);
				CGBDD ();
				~CGBDD();

				void run     ();
				int  onKbhit (int aKey);
};

//-------------CONSTRUCTORS------------------------------------------------\\
//ENTRY:  many usefull parametrs;
//RETURN: object;

CGBDD::CGBDD (int aRoadNum, CRoad **aRoad, int aMaskNum, CCarMask **aMask,
					CPoint<float> aLoc, float aKof, CScreen *aDc, int aBk)
{
if (!aDc) terminate ("Graphic system is not init!\n");
myBackCol = aBk;
myDc      = aDc;
myKof     = aKof;
myLoc     = aLoc;
myRoadNum = aRoadNum;
myMaskNum = aMaskNum;
myParking = new CPark (CRect<int> (0, myDc->maxy() - 30, 300, 30), myBackCol);
myMask    = (CCarMask**) malloc (sizeof (CCarMask*) * myMaskNum);
myRoad    = (CRoad   **) malloc (sizeof (CRoad   *) * myRoadNum);
myCar     = (CCar    **) malloc (sizeof (CCar    *) * myRoadNum);
assert (myMask && myRoad && myCar && myParking);
for (int i = 0; i < myMaskNum; i++) myMask[i] = aMask[i];
randomize();
for (i = 0; i < myRoadNum; i++)
	{
	if (aRoad[i] == NULL) terminate ("it's the CCarManager::CCarManager\n");
	myRoad[i] = aRoad[i];
	myCar [i] = new CCar (myDc, myRoad[i], myMask[random(myMaskNum)],
										myKof, myBackCol, myParking);
	if (myCar[i] == NULL) terminate ("it's the CCarManager::CCarManager\n");
	myRoad[i]->setstart (myLoc);
	}
}

CGBDD::CGBDD()
{
}

CGBDD::~CGBDD()
{
for (int i = 0; i < myRoadNum; i++) { delete myCar[i];  delete myRoad[i]; }
for (    i = 0; i < myMaskNum; i++) { delete myMask[i]; }
delete myDc;
delete myParking;
free (myCar);
free (myRoad);
free (myMask);
}

//-------------FUNCTIONS---------------------------------------------------\\
//-------------runing-car-manager-----------------------------------------\\
//ENTRY:  void;
//RETURN: void;
void CGBDD::run()
{
for (int i = 0; i < myRoadNum; i++) myCar[i]->prepare();
myDraw();
for (;;)
	{
	for (int i = 0; i < myRoadNum; i++)
		{
		if (myCar[i]->move() == ENDMOVE)
			{
			myCar[i]->resetall (myCar[i]->road(),
					myMask [random (myMaskNum)]);
			}
		}
	if (kbhit()) if (onKbhit (bioskey(0)) == END_RUN) return;
	}
}
//-------------handler-that-called-when-a-key-is-pressed-------------------\\
//ENTRY:  a key (in format of bioskey(0));
//RETURN: int parametr (defferent in dif situations);
int CGBDD::onKbhit (int aKey)
{
switch (aKey)
	{
	case ESC : return END_RUN;
	case UP  : myKof += 0.01; myNewKof(); break;
	case DOWN: myKof -= 0.01; myNewKof(); break;
	}
return TRUE;
}

void CGBDD::myNewKof()
{
if (myKof < 0) myKof = 0;
for (int i = 0; i < myRoadNum; i++) myCar[i]->kof (myKof);
}

void CGBDD::myDraw()
{
myDc->fillback (myBackCol);
//myDc->rect (CRect<int> (roundoff(myLoc.x) - 3, roundoff(myLoc.y) - 3, 6, 6),
//																LIGHTGREEN);
}

#endif