#ifndef __CMASK_H__
#define __CMASK_H__

#include <assert.h>
#include <cbutton.h>

class CMask: public CButton
{
protected:
				CPoint<int> myPrevLoc;
				int 	   *myColors;
				int  		myColor;
				int	  	    myHor;
				int		    myVer;
				int		    myColW;
				int		    myColH;

				virtual void click       (int aStat, CPoint<int> aLoc);
				virtual void move        (           CPoint<int> aLoc);
				virtual void newcolor    (int aI, int aJ, int aBut);
				virtual void drawpreview ();
public:
				CMask (CRect<int> aLoc, CPoint<int> aP, int aHor, int aVer);
				CMask ();
				virtual ~CMask();


				virtual int *getarrey  ();
				virtual void setarrey  (int *aMass);
				virtual void getFocus  ();
				virtual void lostFocus ();
				virtual void onMouse   (CPoint<int> aLoc);
				virtual void draw      ();
				virtual void setcolor  (int aColor);
				virtual void offMouse  ();
};

CMask::CMask (CRect<int> aLoc, CPoint<int> aP, int aHor, int aVer):
CButton (aLoc, "")
{
myPrevLoc  = aP;
myColor    = BLACK;
myHor      = aHor;
myVer      = aVer;
myColW     = myLoc.width()  / myHor;
myColH     = myLoc.height() / myVer;
myColors   = new int [myHor * myVer];
assert (myColors);
for (int i = 0; i < myHor * myVer; i++) myColors[i] = NON;
}

CMask::CMask()
{
myHor      = 1;
myVer      = 1;
myColor    = BLACK;
myColors   = new int [myHor * myVer];
assert (myColors);
}

CMask::~CMask()
{
delete myColors;
}

void CMask::draw()
{
if (!myDc) return;
myMouse->hide();
viewport();
myDc->rect3d (myLoc, PRESS);
for (int i = 0; i < myVer; i++)
	for (int j = 0; j < myHor; j++)
		{
		if (myColors [i * myHor + j] == NON)
			myDc->rect (CRect<int> (myLoc.x() + myColW * j,
									myLoc.y() + myColH * i,
									myColW, myColH), LIGHTGRAY, BLACK);
		else
			myDc->rect (CRect<int> (myLoc.x() + myColW * j,
									myLoc.y() + myColH * i,
									myColW, myColH),
									myColors [i * myHor + j]);
		}
drawpreview();
myMouse->show();
}

void CMask::click (int aStat, CPoint<int> aLoc)
{
if (aStat == FALSE)
	if (myStatus != UNPRESS) myClickStat = FALSE;

if ((myMouse->whatOff() == LEFTB || myMouse->whatOff() == RIGHTB) &&
												   !myClickStat)
	myClickStat = TRUE;


if ((myMouse->whatOff() == LEFTB || myMouse->whatOff() == RIGHTB) &&
										   myClickStat == TRUE)
	{
	myClickStat = FALSE;
	newcolor ((aLoc.x - myLoc.x()) / myColW,
			  (aLoc.y - myLoc.y()) / myColH, myMouse->whatOff());
	}
}

void CMask::newcolor (int aI, int aJ, int aBut)
{
myMouse->hide();
viewport();
myDc->addviewport (myLoc);
if (aBut == LEFTB)
	{
	myDc->rect (CRect<int> (myColW * aI, myColH * aJ, myColW, myColH),
													myColor);
	myColors [aJ * myHor + aI] = myColor;
	}

if (aBut == RIGHTB)
	{
	myDc->rect (CRect<int> (myColW * aI, myColH * aJ, myColW, myColH),
													LIGHTGRAY, BLACK);
	myColors [aJ * myHor + aI] = NON;
	}
drawpreview();
myMouse->show();
}

void CMask::setcolor (int aColor)
{
myColor = aColor;
}

void CMask::move (CPoint<int> aLoc)
{
if (myMouse->buttons())
	{
	newcolor ((aLoc.x - myLoc.x()) / myColW,
			  (aLoc.y - myLoc.y()) / myColH, myMouse->buttons());
	}
}

void CMask::onMouse (CPoint<int> aLoc)
{
if   (!myMouse->move() && myStatus != HIDE) click (TRUE, aLoc);
else move (aLoc);
}

void CMask::offMouse ()
{
}

void CMask::getFocus()
{
myMouse->hide();
viewport();
myDc->rect (myLoc.grow(2), NON, BLACK);
myMouse->show();
}

void CMask::lostFocus()
{
myMouse->hide();
viewport();
myDc->rect (myLoc.grow(2), NON, LIGHTGRAY);
myMouse->show();
}

void CMask::drawpreview()
{
myMouse->hide();
viewport();
myDc->rect3d (CRect<int> (myPrevLoc.x, myPrevLoc.y, myHor, myVer), PRESS);
for (int i = 0; i < myHor * myVer; i++)
	{
	if (myColors[i] != NON)
		{
		myDc->point (CPoint<int> (myPrevLoc.x + i % myHor,
								  myPrevLoc.y + i / myHor), myColors[i]);
		}
	}
myMouse->show();
}

int *CMask::getarrey()
{
return myColors;
}

void CMask::setarrey (int *aMass)
{
if (aMass) for (int i = 0; i < myHor * myVer; i++) myColors[i] = aMass[i];
else       for (int i = 0; i < myHor * myVer; i++) myColors[i] = NON;
draw();
}

#endif