#ifndef __CRECT_H_
#define __CRECT_H_

#include <stdio.h>
#include <process.h>

#include <cformat.h>
//------------------------------------------------------------------------\\
void terminate (char *msg);
//----------------------CLASSes-------------------LOCATIN------------------\\
//-----------------------CPoint--------------------------------------------\\
template <class C>
class CPoint
{
public:     C x, y, z;

			CPoint (C aX, C aY, C aZ)
			{
			x = aX;
			y = aY;
			z = aZ;
			};

			CPoint (C aX, C aY)
			{
			x = aX;
			y = aY;
			z = 0;
			};

			CPoint (C val)
			{
			x = y = z = val;
			};

			CPoint ()
			{
			x = y = z = 0;
			};

			CPoint (const CPoint <C>& that)
			{
			x = that.x;
			y = that.y;
			z = that.z;
			};

//--------------------operators-----------------------------------------\\

			CPoint<C>& operator = (const CPoint<C>& that)
			{
			x = that.x;
			y = that.y;
			z = that.z;
			return *this;
			};

			CPoint<C>& operator += (const CPoint<C>& that)
			{
			x += that.x;
			y += that.y;
			z += that.z;
			return *this;
			};

			CPoint<C>& operator -= (const CPoint<C>& that)      	  				//
			{
			x -= that.x;
			y -= that.y;
			z -= that.z;
			return *this;
			};

			CPoint<C>& operator ++ (int)                          				//
			{
			x++;
			y++;
			z++;
			return *this;
			};

			CPoint<C>& operator -- (int)
			{
			x--;
			y--;
			z--;
			return *this;
			};

			CPoint<C> operator + (const CPoint<C>& that) const
			{
			return CPoint<C> (x + that.x, y + that.y, z + that.z);
			};

			CPoint<C> operator - (const CPoint<C>& that)  const
			{
			return CPoint<C> (x - that.x, y - that.y, z - that.z);
			};

			CPoint<C> operator * (C aScale) const
			{
			return CPoint<C> (x * aScale, y * aScale, z * aScale);
			};

			CPoint<C> operator * (const CPoint<C>& that) const
			{
			return CPoint<C> (x * that.x, y * that.y, z * that.z);
			};

			int operator == (const CPoint<C>& that)  const
			{
			return  x == that.x && y == that.y && z == that.z;
			};

			int operator <= (const CPoint<C>& that)  const
			{
			return  x <= that.x && y <= that.y && z <= that.z;
			};

			int operator >= (const CPoint<C>& that)  const
			{
			return  x >= that.x && y >= that.y && z >= that.z;
			};

			int operator != (const CPoint<C>& that)  const
			{
			return  x != that.x || y != that.y || z != that.z;
			};

			int operator < (const CPoint<C>& that)  const
			{
			return  x < that.x && y < that.y && z < that.z;
			};

			int operator > (const CPoint<C>& that)  const
			{
			return  x > that.x && y > that.y && z > that.z;
			};

			int operator ! ()  const
			{
			return  !x && !y && !z;
			};
//----------------------------------------------------------------------//
			const char *typeOfThis() const
			{
			return typeOf (*this);
			}

			int dump (FILE *dumpFile) const
			{
			C Cooler;
			const char *Type = formatStr (Cooler);

			fprintf (dumpFile, typeOfThis());
			fprintf (dumpFile, " <%s> \n{\n", ok()? "ok" : "BAD");

			fprintf (dumpFile, "x = <");
			fprintf (dumpFile, Type, x);
			fprintf (dumpFile, ">\n");

			fprintf (dumpFile, "y = <");
			fprintf (dumpFile, Type, y);
			fprintf (dumpFile, ">\n");

			fprintf (dumpFile, "z = <");
			fprintf (dumpFile, Type, z);
			fprintf (dumpFile, ">\n");

			fprintf (dumpFile, "}\n");
			return ferror (dumpFile);
			};

			int ok () const { return 1; };

};
//------------------------CRect--------------------------------------------\\
template <class C>
class CRect
{
protected:	CPoint<C> myLoc, mySize;


public:     CRect (CPoint<C> newLoc, CPoint<C> newSize) :
				myLoc  (newLoc),
				mySize  (newSize)
			{
			}

			CRect (C aX, C aY, C aZ, C aWidth, C aHeight, C aDepth) :
				myLoc  (aX, aY, aZ),
				mySize (aWidth, aHeight, aDepth)
			{
			}

			CRect (C aX, C aY, C aWidth, C aHeight) :
				myLoc  (aX, aY, 0),
				mySize (aWidth, aHeight, 0)
			{
			}


			CRect (C val) :
				myLoc  (val),
				mySize (val)
			{
			}

			CRect ()
			{
			}

			CRect (const CRect<C>& that) :
				myLoc  (that.myLoc),
				mySize (that.mySize)
			{
			}
//------------------------operators-----------------------------------------\\

			CRect<C>&  operator = (const CRect<C>& that)
			{
			myLoc   = that.myLoc;
			mySize  = that.mySize;
			return *this;
			}

			CRect<C>& operator = (const CPoint<C>& that)
			{
			myLoc = that;
			return *this;
			}

			CRect<C>& operator += (const CRect<C> & that)
			{
			myLoc += that.myLoc;
			mySize += that.mySize;
			return *this;
			}

			CRect<C>& operator += (const CPoint<C> & that)
			{
			myLoc += that;
			return *this;
			}

			CRect<C>& operator -= (const CRect<C>& that)
			{
			myLoc -= that.myLoc;
			mySize -= that.mySize;
			return *this;
			}                                    		  																					//

			CRect<C>& operator -= (const CPoint<C>& that)
			{
			myLoc -= that;
			return *this;
			}

			CRect<C>& operator ++ (int)
			{
			myLoc++;
			mySize++;
			return *this;
			}

			CRect<C>& operator -- (int)
			{
			myLoc--;
			mySize--;
			return *this;
			}

			CRect<C> operator + (const CRect<C>& that) const
			{
			return CRect<C> (myLoc + that.myLoc, mySize + that.mySize);
			}

			CRect<C> operator + (const CPoint<C>& that) const
			{
			return CRect<C>  (myLoc + that, mySize);
			}

			CRect<C> operator - (const CRect<C>& that)  const
			{
			return CRect<C>  (myLoc - that.myLoc,  mySize - that.mySize);
			}

			CRect<C> operator - (const CPoint<C>& that) const
			{
			return CRect<C>  (myLoc - that, mySize);
			}

			CRect<C> operator * (C aScale) const
			{
			return CRect<C>  (myLoc * aScale,  mySize * aScale);
			}

			CRect<C> operator * (const CRect<C>& that) const
			{
			return CRect<C>  (myLoc * that.myLoc,  mySize * that.mySize);
			}

			CRect<C> operator * (const CPoint<C>& that) const
			{
			return CRect<C>  (myLoc * that, mySize);
			}

			int operator == (const CRect<C>& that)  const
			{
			return myLoc == that.myLoc && mySize == that.mySize;
			}

			int operator != (const CRect<C>& that)  const
			{
			return myLoc != that.myLoc || mySize != that.mySize;
			}

			int operator <= (const CRect<C>& that)  const
			{
			return myLoc <= that.myLoc && mySize <= that.mySize;
			}

			int operator >= (const CRect<C>& that)  const
			{
			return myLoc >= that.myLoc && mySize >= that.mySize;
			}

			int operator < (const CRect<C>& that)  const
			{
			return myLoc < that.myLoc && mySize < that.mySize;
			}

			int operator > (const CRect<C>& that)  const
			{
			return myLoc > that.myLoc && mySize > that.mySize;
			}


			int operator == (const CPoint<C>& that)  const
			{
			return myLoc == that;
			}

			int operator != (const CPoint<C>& that)  const
			{
			return myLoc != that;
			}

			int operator <= (const CPoint<C>& that)  const
			{
			return myLoc <= that;
			}

			int operator >= (const CPoint<C>& that)  const
			{
			return myLoc >= that;
			}

			int operator < (const CPoint<C>& that)  const
			{
			return myLoc < that;
			}

			int operator > (const CPoint<C>& that)  const
			{
			return myLoc > that;
			}

			int operator ! ()  const
			{
			return  !myLoc  && !mySize;
			};

//--------------------------------------------------------------------------//
			void x (C aX)      	 { myLoc.x = aX; };
			void y (C aY)      	 { myLoc.y = aY; };
			void z (C aZ)      	 { myLoc.z = aZ; };

			C x ()  const 		 { return myLoc.x ; };
			C y ()  const		 { return myLoc.y ; };
			C z ()  const	     { return myLoc.z ; };

			void width  (C aWidth)  { mySize.x = aWidth;  };
			void height (C aHeight) { mySize.y = aHeight; };
			void depth  (C aDepth)  { mySize.z = aDepth;  };
			C width  () const		 { return mySize.x; };
			C height () const		 { return mySize.y; };
			C depth  () const		 { return mySize.z; };

			void x2 (C aX)			{ mySize.x = aX - myLoc.x;};
			void y2 (C aY)			{ mySize.y = aY - myLoc.y;};
			void z2 (C aZ)			{ mySize.z = aZ - myLoc.z;};

			C x2 ()  const			{ return   myLoc.x + mySize.x;};
			C y2 ()  const			{ return   myLoc.y + mySize.y;};
			C z2 ()  const			{ return   myLoc.z + mySize.z;};

			C centerX ()  const	 { return myLoc.x + mySize.x/2;  };
			C centerY ()  const	 { return myLoc.y + mySize.y/2;  };
			C centerZ ()  const	 { return myLoc.z + mySize.z/2;  };

			int contain3d	(CPoint<C>& aLoc)
			{ return aLoc >= myLoc && aLoc <= (myLoc + mySize); };

			int contain	(CPoint<C> aLoc)
			{
			return aLoc.x  >= myLoc.x  &&
				   aLoc.y  >= myLoc.y  &&
				   aLoc.x  <= x2()     &&
				   aLoc.y  <= y2();
			}

			CRect<C> grow (C aBig)
			{
			return CRect<C> (x() - aBig, y() - aBig,
							 width() + 2*aBig, height() + 2*aBig);
			}

//--------------JUSTIFY-----------------------------------------------------\\
			CRect left  () const
			{
			return CRect<C> (myLoc.x,
							 myLoc.y,
							 myLoc.z,
							 0,
							 mySize.y,
							 mySize.z);
			}

			CRect right  () const
			{
			return CRect<C> (myLoc.x + mySize.x,
							 myLoc.y,
							 myLoc.z,
							 0,
							 mySize.y,
							 mySize.z);
			}

			CRect top  ()  const
			{
			return CRect<C> (myLoc.x,
							 myLoc.y,
							 myLoc.z,
							 mySize.x,
							 0,
							 mySize.z);
			}

			CRect bottom  ()  const
			{
			return CRect<C> (myLoc.x,
							 myLoc.y + mySize.y,
							 myLoc.z,
							 mySize.x,
							 0,
							 mySize.z);
			}

			CRect front  ()   const
			{
			return CRect<C> (myLoc.x,
							 myLoc.y,
							 myLoc.z,
							 mySize.y,
							 mySize.x,
							 0);
			}

			CRect back  ()  const
			{
			return CRect<C> (myLoc.x,
							 myLoc.y,
							 myLoc.z + mySize.z,
							 mySize.x,
							 mySize.y,
							 mySize.z);
			}
//--------------------------------------------------------------------------\\
			const char *typeOfThis() const
			{
			return typeOf(*this);
			}

			int dump (FILE *dumpFile)  const
			{
			C Cooler;
			const char *Type = formatStr (Cooler);

			fprintf (dumpFile, typeOfThis());
			fprintf (dumpFile, " <%s> \n{\n", ok()? "ok" : "BAD");

			fprintf (dumpFile, "x      = <");
			fprintf (dumpFile, Type , myLoc.x);
			fprintf (dumpFile, ">\n");

			fprintf (dumpFile, "y      = <");
			fprintf (dumpFile, Type, myLoc.y);
			fprintf (dumpFile, ">\n");

			fprintf (dumpFile, "z      = <");
			fprintf (dumpFile, Type, myLoc.z);
			fprintf (dumpFile, ">\n");

			fprintf (dumpFile, "width  = <");
			fprintf (dumpFile, Type, mySize.x);
			fprintf (dumpFile, ">\n");

			fprintf (dumpFile, "height = <");
			fprintf (dumpFile, Type, mySize.y);
			fprintf (dumpFile, ">\n");

			fprintf (dumpFile, "depth  = <");
			fprintf (dumpFile, Type, mySize.z);
			fprintf (dumpFile, ">\n");

			fprintf (dumpFile, "}\n");
			return ferror (dumpFile);
			}

			int ok () const
			{              //!!VVV THREE wornings becouse of this!!!
			return 1; //mySize.x >= 0 && mySize.y >= 0 && mySize.z >= 0;
			}              //!!^^^
}

const CRect<int>  FULL (0, 0, 0, 0);
const CPoint<int> START (0, 0);
//==========================================================================
void terminate (char *msg)
{
fprintf (stderr, msg);
abort();
}

#endif