#ifndef __CROAD_H__
#define __CROAD_H__

#include <graphics.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <assert.h>
#include <math.h>
#include <dos.h>

#include <crect.h>
#include <ccarmask.h>
#include <cmath.h>


#define 			MAX_SPEED_LIMITS			4

CCarMask GBDDMask [MAX_SPEED_LIMITS] =
	{
	("gbdd\\gbdd35.car"),
	("gbdd\\gbdd60.car"),
	("gbdd\\gbdd90.car"),
	("gbdd\\gbdd130.car"),
	};

const int SpeedLimits[MAX_SPEED_LIMITS] = { 35, 60, 90, 130 };

class CRoad
{
protected:
				CPoint<float> *mySpots;
				CPoint<int>   *myGBDD;
				int            myGBDDNum;
				int            mySpotNum;
				int            myColor;


				virtual void	myLoad     (char *file);
				virtual void	myGBDDLoad (FILE *f);
public:
				CRoad (int aSpotNum, CPoint<float> *aSpots, int aCol = RED);
				CRoad (char *file);

				virtual CPoint<float> getpos (float aWay);
				virtual void draw     ();
				virtual void setstart (CPoint<float> aLoc);
				virtual  int testGBDD (float aWay, float aSpeed, int &aGBDD);

				virtual void		  dump     (FILE *f);
};

CRoad::CRoad (int aSpotNum, CPoint<float> *aSpots, int aCol)
{
myGBDDNum   = 0;
mySpotNum   = aSpotNum;
myGBDD      = NULL;
myColor     = aCol;
mySpots     = new CPoint<float> [mySpotNum + 1];
assert (mySpots);
for (int i = 0; i < mySpotNum; i++) mySpots[i+1] = aSpots[i];
}

CRoad::CRoad (char *file)
{
myLoad (file);
}

void CRoad::myLoad (char *file)
{
FILE *f = fopen (file, "r");
if (!f) terminate ("CRoad file not found!\n");
char tmp[10];
fscanf (f, "%s", tmp);
if (strcmp ("CRoad", tmp)) terminate ("illegal format");
fscanf (f, "%d", &mySpotNum);
if (mySpotNum++ <= 0) terminate ("illegal format");
mySpots = new CPoint<float> [mySpotNum];
assert (mySpots);
for (int i = 0; i < mySpotNum - 1; i++)
	fscanf (f, "%f %f", &(mySpots[i+1].x), &(mySpots[i+1].y));
fscanf (f, "%d", &myColor);
myGBDDLoad (f);
fclose (f);
}

void CRoad::myGBDDLoad (FILE *f)
{
char tmp[10];
fscanf (f, "%s", tmp);
if (strcmp (tmp, "GBDD")) { myGBDDNum = 0; myGBDD = NULL; return; }
fscanf (f, "%d", &myGBDDNum);
if (myGBDDNum < 0) terminate ("illegal format");
myGBDD = new CPoint<int> [myGBDDNum];
assert (myGBDD);
for (int i = 0; i < myGBDDNum; i++)
	fscanf (f, "%d %d", &(myGBDD[i].x), &(myGBDD[i].y));
}


void CRoad::draw()
{
for (int i = 0; i < mySpotNum - 1; i++)
	{
	setcolor (myColor);
	line (roundoff (mySpots[i].x),   roundoff (mySpots[i].y),
		  roundoff (mySpots[i+1].x), roundoff (mySpots[i+1].y));
	}
if (myGBDDNum)
	for (int i = 0; i < myGBDDNum; i++)
		{
		CPoint<float> tmp = getpos (myGBDD[i].y);
		int aNum = myGBDD[i].x;
		if (aNum >= MAX_SPEED_LIMITS) aNum = MAX_SPEED_LIMITS - 1;
		GBDDMask [aNum].draw (CPoint<float>
			(tmp.x - HORIZ_DOT, tmp.y - VERT_DOT, -M_PI/2), DRAW, BLACK);
		}
}

void CRoad::setstart (CPoint<float> aLoc)
{
mySpots[0] = aLoc;
}

int CRoad::testGBDD (float aWay, float aSpeed, int &aGBDD)
{
if (aGBDD >= myGBDDNum) return FALSE;
if (aWay >= myGBDD[aGBDD].y)
	{
	int myLimType  = myGBDD[aGBDD++].x;
	if (myLimType >= MAX_SPEED_LIMITS)   return FALSE;
	if (aSpeed > SpeedLimits[myLimType]) return TRUE;
	}
return FALSE;
}


CPoint<float> CRoad::getpos (float aWay)
{
CPoint<float> res;
float l = 0;
float a, b, c;
for (int i = 0; i < mySpotNum - 1; i++)
	{
	a = mySpots[i+1].x - mySpots[i].x;
	b = mySpots[i+1].y - mySpots[i].y;
	c = hypot (a, b);
	l += c;
	if (aWay <= l)
		{
		float d = sqrt (pow ((b/c), 2));
		if (c)
			{
			if ((a <= 0) && (b <= 0)) res.z = asin (d) + M_PI;
			if ((a >  0) && (b >  0)) res.z = asin (d);
			if ((a >  0) && (b <= 0)) res.z = 2*M_PI - asin (d);
			if ((a <= 0) && (b >  0)) res.z =   M_PI - asin (d);

			res.x = roundoff (mySpots[i].x + (aWay - l + c) * cos (res.z));
			res.y = roundoff (mySpots[i].y + (aWay - l + c) * sin (res.z));
		   }
		return res;
		}
	}
return CPoint<float> (0xFF, 0xFF, 0xFF);
}

void CRoad::dump (FILE *f)
{
fprintf (f, "%d\n", mySpotNum);
}

#endif
